# Copyright (C) 2018  Nexedi SA
#     Lukasz Nowak <luke@nexedi.com>
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.

import argparse
import httplib
import requests
import sys

import app


def http(*args):
  if not args:
    args = sys.argv[1:]

  parser = argparse.ArgumentParser(description='KeDiFa webapp')
  parser.add_argument(
    '--ip',
    help='IP to bind to (v4 or v6).',
    required=True
  )
  parser.add_argument(
    '--port',
    type=int,
    help='Port to bind to.',
    required=True
  )
  parser.add_argument(
    '--db',
    help='Path to the SQLite database.',
    required=True
  )
  parser.add_argument(
    '--certificate',
    type=argparse.FileType('r'),
    help='Path SSL certificate.',
    required=True
  )
  parser.add_argument(
    '--ca-certificate',
    type=argparse.FileType('r'),
    help='Path SSL CA certificate.',
    required=True
  )
  parser.add_argument(
    '--crl',
    type=argparse.FileType('r'),
    help='Path SSL CRL.',
    required=True
  )
  parser.add_argument(
    '--pidfile',
    type=argparse.FileType('w'),
    help='Path to PID file.',
    required=True
  )
  parsed = parser.parse_args(args)

  app.http(parsed.ip, parsed.port, parsed.db, parsed.certificate,
           parsed.ca_certificate, parsed.crl, parsed.pidfile)


def getter(*args):
  """Downloads a file with SSL client & server authentication, storing
     received data only when successful.
  """
  if not args:
    args = sys.argv[1:]

  parser = argparse.ArgumentParser(description='KeDiFa getter')
  parser.add_argument(
    'url',
    nargs=1,
    help='URL to fetch certificate from.'
  )
  parser.add_argument(
    '--identity',
    type=argparse.FileType('r'),
    help='Certificate to identify itself on the URL.',
    required=True
  )
  parser.add_argument(
    '--server-ca-certificate',
    type=argparse.FileType('r'),
    help='CA Certificate of the server.',
    required=True
  )
  parser.add_argument(
    '--out',
    help='Destination to store the downloaded certificate.',
    required=True
  )
  parsed = parser.parse_args(args)

  url = parsed.url[0]
  try:
    response = requests.get(url, verify=parsed.server_ca_certificate.name,
                            cert=parsed.identity.name)
  except Exception as e:
    print '%r not downloaded, problem %s' % (url, e)
    sys.exit(1)
  else:
    if response.status_code != httplib.OK:
      print '%r not downloaded, HTTP code %s' % (
        url, response.status_code)
      sys.exit(1)
    if len(response.text) > 0:
      with open(parsed.out, 'w') as out:
        out.write(response.text.encode('utf-8'))
